/*
 * Playground+ - Draughts.c
 * Draughts code v1.1 (by Radiac)
 *    http://radiac.net/pgplus
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>

#include "include/config.h"
#include "include/player.h"
#include "include/proto.h"

void draughts_display(player *p);

draughts_board *draughts_board_create(player *p, player *p2) {
	int x,y;
	draughts_board *board;
	board = (draughts_board *)malloc(sizeof(draughts_board));
	board->a = p;
	board->b = p2;
	board->next = p2;
	board->prev = 0;
	board->oxs = -1;
	board->oys = -1;
	board->oxt = -1;
	board->oyt = -1;
	board->moved = 0;	/* 0=start  1=move  2=jump  3=huff  4=win */
	for (x=0; x<8; x++) {
		for (y=0; y<8; y++) {
			if ((x+y)%2 == 0) {
				board->square[x][y] = 0;
			} else if (y<3) {
				board->square[x][y] = DRAUGHTS_PLAYER_A;
			} else if (y>4) {
				board->square[x][y] = DRAUGHTS_PLAYER_B;
			} else {
				board->square[x][y] = 0;
			}
		}
	}
	return board;
}

player *draughts_find_opponent(player *p) {
	if (p->draughts_board->a == p)
		return p->draughts_board->b;
	else
		return p->draughts_board->a;
}

int draughts_find_flag(player * p) {
	if (p->draughts_board->a == p)
		return DRAUGHTS_PLAYER_A;
	else
		return DRAUGHTS_PLAYER_B;
}
int draughts_find_opponent_flag(player *p) {
	/* Can't use previous two subs together, in case the player disconnected */
	if (p->draughts_board->a == p)
		return DRAUGHTS_PLAYER_B;
	else
		return DRAUGHTS_PLAYER_A;
}

int draughts_read_coords(char *var, int *x, int *y) {
	*var = tolower(*var);
	*(var+1) = tolower(*(var+1));
	if (*var - 'a' >= 0 && *var - 'a' <= 7) {
		*x = *var++ - 'a';
		if (*var - '1' >= 0 && *var - '1' <= 7)
			*y = *var - '1';
		else
			*x = -1;
	} else if (*var - '1' >= 0 && *var - '1' <= 7) {
		*y = *var++ - '1';
		if (*var - 'a' >= 0 && *var - 'a' <= 7)
			*x = *var - 'a';
		else
			*x = -1;
	} else {
		*x = -1;
	}

	if (*x == -1)
		return 0;
	else
		return 1;
}

int draughts_can_jump(player *p, int x, int y) {
	int op_flag;
	if (p->draughts_board->square[x][y] == 0)
		return 0;
	if (p->draughts_board->square[x][y] & DRAUGHTS_KING) {
		op_flag = draughts_find_opponent_flag(p);
		if ((x-2>=0 && y+2<8 && p->draughts_board->square[x-1][y+1] & op_flag && p->draughts_board->square[x-2][y+2] == 0) ||
			(x+2>=0 && y+2<8 && p->draughts_board->square[x+1][y+1] & op_flag && p->draughts_board->square[x+2][y+2] == 0) ||
			(x-2>=0 && y-2>=0 && p->draughts_board->square[x-1][y-1] & op_flag && p->draughts_board->square[x-2][y-2] == 0) ||
			(x+2>=0 && y-2>=0 && p->draughts_board->square[x+1][y-1] & op_flag && p->draughts_board->square[x+2][y-2] == 0))
			return 1;
	} else if (p->draughts_board->square[x][y] & DRAUGHTS_PLAYER_A) {
		/* Moving down */
		if ((x-2>=0 && y+2<8 && p->draughts_board->square[x-1][y+1] & DRAUGHTS_PLAYER_B && p->draughts_board->square[x-2][y+2] == 0) ||
			(x+2>=0 && y+2<8 && p->draughts_board->square[x+1][y+1] & DRAUGHTS_PLAYER_B && p->draughts_board->square[x+2][y+2] == 0))
			return 1;
	} else if (p->draughts_board->square[x][y] & DRAUGHTS_PLAYER_B) {
		/* Moving up */
		if ((x-2>=0 && y-2>=0 && p->draughts_board->square[x-1][y-1] & DRAUGHTS_PLAYER_A && p->draughts_board->square[x-2][y-2] == 0) ||
			(x+2>=0 && y-2>=0 && p->draughts_board->square[x+1][y-1] & DRAUGHTS_PLAYER_A && p->draughts_board->square[x+2][y-2] == 0))
			return 1;
	}
	return 0;
}

void draughts_disconnect(player *p) {
	/* p has disconnected */
	if (p->draughts_board->a && p->draughts_board->b) {
		/* One person has left a game */
		TELLPLAYER(draughts_find_opponent(p), " %s has disconnected. 'draughts invite <player>' will invite someone to pick up where %s left off.\n", p->name, gstring(p));
		if (p->draughts_board->a == p)
			p->draughts_board->a = 0;
		else
			p->draughts_board->b = 0;
		if (p->draughts_board->next == p)
			p->draughts_board->next = 0;
	} else {
		/* Both people have left the game */
		free(p->draughts_board);
		p->draughts_board = 0;
	}
}


/****************/
/*** Commands ***/
/****************/

void draughts_invite(player *p, char *var) {
	player *p2;
	if (!*var) {
		TELLPLAYER(p, " Format: draughts invite <player>\n");
		return;
	}
	p2 = find_player_global(var);
	if (!p2) {
		/* p2 is not a player, or is at least not logged on. p is informed automatically */
		return;
	}
	if (p == p2) {
		TELLPLAYER(p, " You can't play yourself!\n");
		return;
	}
	if (p2->draughts_board) {
		TELLPLAYER(p, " %s is already playing draughts.\n", p2->name);
		return;
	}

	if (p->draughts_board) {
		/* We have a board, see if it's active */
		if (p->draughts_board->a && p->draughts_board->b) {
			TELLPLAYER(p, " But you are already playing a game of draughts!\n");
			return;
		}

		/* Player is OK, add them to the board */
		if (p->draughts_board->a) {
			p->draughts_board->b = p2;
		} else {
			p->draughts_board->a = p2;
		}
		if (!p->draughts_board->next)
			p->draughts_board->next = p2;
		p2->draughts_board = p->draughts_board;

		/* Tell players */
		TELLPLAYER(p, " Continuing game with %s...\n", p2->name);
		TELLPLAYER(p2, " %s has invited you to continue %s game of draughts!\n Type 'draughts quit' to decline.\n", p->name, gstring_possessive(p));

	} else {
		/* Can just start */
		p->draughts_board = draughts_board_create(p, p2);
		p2->draughts_board = p->draughts_board;

		/* Tell players */
		TELLPLAYER(p, " Starting game against %s...\n", p2->name);
		TELLPLAYER(p2, " %s has challenged you to a game of draughts!\n Type 'draughts quit' to decline.\n", p->name);
	}

	/* Display the board to both */
	draughts_display(p);
	draughts_display(p2);
}

void draughts_quit(player *p) {
	player *p2;

	if (!p->draughts_board) {
		TELLPLAYER(p, " But you're not playing draughts!\n");
		return;
	}

	/* We're playing draughts */
	if (p->draughts_board->a && p->draughts_board->b) {
		/* Someone else is playing, find out if we're resigning */
		p2 = draughts_find_opponent(p);

		/* Tell them whats happening */
		if (p->draughts_board->moved == 0) {
			if (p->draughts_board->a == p) {
				TELLPLAYER(p, " You withdraw your invitation to play draughts.\n");
				if (p2) TELLPLAYER(p2, " %s has withdrawn %s invitation to play draughts.\n", p->name, gstring_possessive(p));
			} else {
				TELLPLAYER(p, " You decline %s invitation to play draughts.\n", gstring_possessive(p2));
				if (p2) TELLPLAYER(p2, " %s declined your invitation to play draughts.\n", p->name);
			}
		} else {
			TELLPLAYER(p, " You resign from the game and declare %s the winner.\n", p2->name);
			if (p2) TELLPLAYER(p2, " %s has resigned from the game. You are the winner!\n", p->name);
			/* Scores? */
		}

		/* Clear p2 */
		if (p2) p2->draughts_board = 0;

	} else {
		/* Nobody else is playing */
		TELLPLAYER(p, " You close your game of draughts.\n");
	}

	/* Stop the game and clear p1 */
	free(p->draughts_board);
	p->draughts_board = 0;
}

void draughts_display(player *p) {
	int x,y;
	char *tstack, *ostack, go;
	tstack = (char *) malloc (40);

	ostack = tstack;
	go = ' ';

	if (p==0)
		return; /* They've disconnected */
	if (!p->draughts_board) {
		TELLPLAYER(p, " But you're not playing draughts!\n");
		return;
	}

	TELLPLAYER(p, "\n^g    A B C D E F G H^N");
	for (y=0; y<8; y++) {
		sprintf(tstack, " ^g%d^c ", y+1);
		tstack = strchr(tstack, 0);
		for (x=0; x<8; x++) {
			if ((x+y)%2 == 0) {
				sprintf(tstack," +");
			} else {
				if (p->draughts_board->square[x][y] & DRAUGHTS_PLAYER_A) {
					if (p->draughts_board->square[x][y] & DRAUGHTS_KING)
						sprintf(tstack, " ^PA^c");
					else
						sprintf(tstack, " ^Pa^c");
				} else if (p->draughts_board->square[x][y] & DRAUGHTS_PLAYER_B) {
					if (p->draughts_board->square[x][y] & DRAUGHTS_KING)
						sprintf(tstack, " ^YB^c");
					else
						sprintf(tstack, " ^Yb^c");
				} else {
					sprintf(tstack," +");
				}
			}
			tstack = strchr(tstack, 0);
		}
		TELLPLAYER(p, "\n%s", ostack);
		if (y==3) {
			if (p->draughts_board->next == p->draughts_board->a)
				go = '*';
			if (p->draughts_board->a)
				TELLPLAYER(p, " ^G%c ^Pa: %s^c", go, p->draughts_board->a->name);
			else
				TELLPLAYER(p, " ^G%c ^Pa:^c", go);
			go = ' ';
		}
		if (y==4) {
			if (p->draughts_board->next == p->draughts_board->b)
				go = '*';
			if (p->draughts_board->b)
				TELLPLAYER(p, " ^G%c ^Yb: %s^c", go, p->draughts_board->b->name);
			else
				TELLPLAYER(p, " ^G%c ^Yb:^c", go);
		}
		tstack = ostack;
	}
	free(ostack);

	TELLPLAYER(p, "^N\n\n");
}

void draughts_move(player *p, char *str, char *var) {
	/* str and var will be in the format 'zz', where z is an x or y coord */
	int xs, ys, xt, yt, fa, fb, tx, ty;

	fa = 0;
	fb = 0;

	/* Perform checks */
	if (!p->draughts_board) {
		TELLPLAYER(p, " But you're not playing draughts!\n");
		return;
	}
	if (p->draughts_board->next != p) {
		TELLPLAYER(p, " But it is not your move!\n");
		return;
	}

	/* Work out the moves */
	if (!draughts_read_coords(str, &xs, &ys)) {
		TELLPLAYER(p, " Format: draughts <piece coords> <target coords>\n");
		return;
	}
	if (!draughts_read_coords(var, &xt, &yt)) {
		TELLPLAYER(p, " Format: draughts <piece coords> <target coords>\n");
		return;
	}

	/* See if it's a valid move */
	if (p->draughts_board->prev == p->draughts_board->next) {
		if (p->draughts_board->oxt != xs || p->draughts_board->oyt != ys) {
			TELLPLAYER(p, " You can only move the piece in %c%d.\n", p->draughts_board->oxt + 'a', p->draughts_board->oyt+1);
			return;
		}
	}
	if (!(p->draughts_board->square[xs][ys] & draughts_find_flag(p))) {
		TELLPLAYER(p, " You do not have a piece there to move.\n");
		return;
	}

	if (p->draughts_board->square[xs][ys] & DRAUGHTS_KING) {
		if (p->draughts_board->square[xt][yt]==0 && abs(yt-ys)==1 && abs(xt-xs)==1) {
			/* step to gap */
			p->draughts_board->square[xs][ys] = 0;
			p->draughts_board->square[xt][yt] = draughts_find_flag(p) | DRAUGHTS_KING;
			p->draughts_board->moved = 1;
		} else if (p->draughts_board->square[xt][yt]==0 && p->draughts_board->square[(xs+xt)/2][(ys+yt)/2] & draughts_find_opponent_flag(p) && abs(yt-ys)==2 && abs(xt-xs)==2) {
			/* jump */
			p->draughts_board->square[xs][ys] = 0;
			p->draughts_board->square[(xs+xt)/2][(ys+yt)/2] = 0;
			p->draughts_board->square[xt][yt] = draughts_find_flag(p) | DRAUGHTS_KING;
			p->draughts_board->moved = 2;
		} else {
			TELLPLAYER(p, " That is not a valid move.\n");
			return;
		}
	} else if (p->draughts_board->square[xs][ys] & DRAUGHTS_PLAYER_A) {
		/* Normal piece moving down */
		if (p->draughts_board->square[xt][yt]==0 && yt==ys+1 && abs(xt-xs)==1) {
			/* step to gap */
			p->draughts_board->square[xs][ys] = 0;
			p->draughts_board->square[xt][yt] = DRAUGHTS_PLAYER_A;
			p->draughts_board->moved = 1;
		} else if (p->draughts_board->square[xt][yt]==0 && p->draughts_board->square[(xs+xt)/2][ys+1] & draughts_find_opponent_flag(p) && yt==ys+2 && abs(xt-xs)==2) {
			/* jump */
			p->draughts_board->square[xs][ys] = 0;
			p->draughts_board->square[(xs+xt)/2][ys+1] = 0;
			p->draughts_board->square[xt][yt] = DRAUGHTS_PLAYER_A;
			p->draughts_board->moved = 2;
		} else {
			TELLPLAYER(p, " That is not a valid move.\n");
			return;
		}

		/* Make a king? */
		if (yt == 7)
			p->draughts_board->square[xt][yt] |= DRAUGHTS_KING;

	} else {
		/* Normal piece moving up */
		if (p->draughts_board->square[xt][yt]==0 && yt==ys-1 && abs(xt-xs)==1) {
			/* step to gap */
			p->draughts_board->square[xs][ys] = 0;
			p->draughts_board->square[xt][yt] = DRAUGHTS_PLAYER_B;
			p->draughts_board->moved = 1;
		} else if (p->draughts_board->square[xt][yt]==0 && p->draughts_board->square[(xs+xt)/2][ys-1] & draughts_find_opponent_flag(p) && yt==ys-2 && abs(xt-xs)==2) {
			/* jump */
			p->draughts_board->square[xs][ys] = 0;
			p->draughts_board->square[(xs+xt)/2][ys-1] = 0;
			p->draughts_board->square[xt][yt] = DRAUGHTS_PLAYER_B;
			p->draughts_board->moved = 2;
		} else {
			TELLPLAYER(p, " That is not a valid move.\n");
			return;
		}

		/* Make a king? */
		if (yt == 0)
			p->draughts_board->square[xt][yt] |= DRAUGHTS_KING;
	}

	/* We've moved */
	p->draughts_board->prev = p;

	/* Check for a win */
	fa = 0; fb = 0;
	for (tx=0; tx<8; tx++) {
		for (ty=0; ty<8; ty++) {
			if (p->draughts_board->square[tx][ty] & DRAUGHTS_PLAYER_A)
				fa = 1;
			if (p->draughts_board->square[tx][ty] & DRAUGHTS_PLAYER_B)
				fb = 2;
		}
	}

	if (!fa || !fb) {
		if (draughts_find_opponent(p)) TELLPLAYER(draughts_find_opponent(p), "^Y %s has won!\n", p->name);
		TELLPLAYER(p, "^Y You have won!\n");
		p->draughts_board->moved = 4;
		p->draughts_board->next = 0;
	} else {

		/* If we jumped, remember what we just did (for multiple jumps and huffs */
		p->draughts_board->oxs = xs;
		p->draughts_board->oys = ys;
		p->draughts_board->oxt = xt;
		p->draughts_board->oyt = yt;

		/* And are there any more jumps for this piece? */
		if (!(p->draughts_board->moved==2 && draughts_can_jump(p, xt, yt) == 1)) {
			/* Swap players */
			p->draughts_board->next = draughts_find_opponent(p);
		}

	}

	/* Display board to both */
	draughts_display(p->draughts_board->a);
	draughts_display(p->draughts_board->b);

	/* If there was a win, delete the game */
	if (!fa || !fb) {
		if (draughts_find_opponent(p)) draughts_find_opponent(p)->draughts_board = 0;
		free(p->draughts_board);
		p->draughts_board = 0;
	}
}

void draughts_huff(player *p, char *var) {
	int x, y;

	/* Run checks */
	if (strlen(var) != 2) {
		TELLPLAYER(p, " Format: draughts huff <piece coords>\n");
		return;
	}
	if (!p->draughts_board) {
		TELLPLAYER(p, " But you're not playing draughts!\n");
		return;
	}
	if (p->draughts_board->next != p) {
		TELLPLAYER(p, " But it's not your go!\n");
		return;
	}
	if (p->draughts_board->moved == 3) {
		TELLPLAYER(p, " You can only huff one piece.\n");
		return;
	}
	if (p->draughts_board->prev == p) {
		TELLPLAYER(p, " You can't do that, that'd be cheating!\n");
		return;
	}
	if (p->draughts_board->moved != 1) {
		TELLPLAYER(p, " You cannot huff that piece.\n");
		return;
	}

	/* Work out the moves */
	if (!draughts_read_coords(var, &x, &y)) {
		TELLPLAYER(p, " Format: draughts huff <piece coords>\n");
		return;
	}

	/* Let's just check that this piece is the opponents */
	if (p->draughts_board->square[x][y] == 0) {
		/* It's empty */
		TELLPLAYER(p, " There is no piece there to huff!\n");
		return;
	}
	if (p->draughts_board->square[x][y] & draughts_find_flag(p)) {
		/* It's the players */
		TELLPLAYER(p, " You cannot huff that piece.\n");
		return;
	}

	/* Roll the board back */
	p->draughts_board->square[p->draughts_board->oxs][p->draughts_board->oys] = p->draughts_board->square[p->draughts_board->oxt][p->draughts_board->oyt];
	p->draughts_board->square[p->draughts_board->oxt][p->draughts_board->oyt] = 0;

	/* Roll back the piece too, if it needs rolling back */
	if (x == p->draughts_board->oxt && y == p->draughts_board->oyt) {
		x = p->draughts_board->oxs;
		y = p->draughts_board->oys;
	}

	/* If this piece could have jumped, huff it */
	if (draughts_can_jump(p, x, y) == 1) {
		p->draughts_board->square[x][y] = 0;
		p->draughts_board->moved = 3;
		TELLPLAYER(p, " You huff the piece.\n");
		if (draughts_find_opponent(p))
			TELLPLAYER(draughts_find_opponent(p), " %s huffs your piece.\n", p->name);
	} else {
		TELLPLAYER(p, " You cannot huff that piece.\n");
	}

	/* Roll the board forwards again */
	p->draughts_board->square[p->draughts_board->oxt][p->draughts_board->oyt] = p->draughts_board->square[p->draughts_board->oxs][p->draughts_board->oys];
	p->draughts_board->square[p->draughts_board->oxs][p->draughts_board->oys] = 0;

	/* Display the board if there was a huff */
	if (p->draughts_board->moved == 3) {
		draughts_display(p->draughts_board->a);
		draughts_display(p->draughts_board->b);
	}
}

void draughts(player *p, char *str) {
	char *var;

	/* Force to lowercase */
	lower_case(str);

	/* Find variable for commands (if present) */
	var = next_space(str);
	*var++ = '\0';

	if (!*str) {
		TELLPLAYER(p, " Format: draughts <option>\n         See 'help draughts' for more information\n");
		return;

	} else if (!strcmp(str, "invite")) {
		draughts_invite(p, var);

	} else if (!strcmp(str, "quit")) {
		draughts_quit(p);

	} else if (!strcmp(str, "view")) {
		draughts_display(p);

	} else if (!strcmp(str, "huff")) {
		draughts_huff(p, var);

	} else if (strlen(str) == 2 && strlen(var) == 2) {
		draughts_move(p, str, var);

	} else {
		TELLPLAYER(p, " Option not recognised\n         See 'help draughts' for more information\n");
		return;
	}
}



/*******************/
/*** Reboot code ***/
/*******************/

void reboot_save_draughts(void) {
	player *scan;
	char *tstack, *ostack, *a, *b, e;
	int x,y;
	FILE *reboot_draughts_file;

	e = 0;
	tstack = (char *)malloc(7+64+1);
	ostack = tstack;

	reboot_draughts_file = fopen("junk/rebooting/draughts", "w");

	if (!reboot_draughts_file) {
		log("reboot", "Couldn't open junk/rebooting/draughts for writing");
		return;
	}
	/* Step through the players */
	for (scan = flatlist_start; scan; scan = scan->flat_next) {
		if (!scan->draughts_board)
			continue; /* They're not playing */
		if ((scan->draughts_board->a && !scan->draughts_board->a->draughts_board) || (scan->draughts_board->b && !scan->draughts_board->b->draughts_board)) {
			/* The board has already been saved. Free, clear and skip */
			free(scan->draughts_board);
			scan->draughts_board = 0;
			continue;
		}

  		/* Here's a draughts board that needs saving */
  		/* Make up a string of the board */
  		if (scan->draughts_board->next == scan->draughts_board->a)
  			*tstack++ = '1';
  		else
  			*tstack++ = '2';

  		if (scan->draughts_board->prev == 0)
  			*tstack++ = '0';
  		else if (scan->draughts_board->prev == scan->draughts_board->a)
  			*tstack++ = '1';
  		else
  			*tstack++ = '2';

		*tstack++ = scan->draughts_board->moved + '0';
		*tstack++ = scan->draughts_board->oxs + '0';
		*tstack++ = scan->draughts_board->oys + '0';
		*tstack++ = scan->draughts_board->oxt + '0';
		*tstack++ = scan->draughts_board->oyt + '0';

  		for (x=0; x<8; x++) {
			for (y=0; y<8; y++) {
				*tstack++ = scan->draughts_board->square[x][y] + '0';
			}
		}
		*tstack = 0;


		if (scan->draughts_board->a) a = scan->draughts_board->a->name; else a = &e;
		if (scan->draughts_board->b) b = scan->draughts_board->b->name; else b = &e;

		fprintf(reboot_draughts_file, "%s %s %s\n", a, b, ostack);
		scan->draughts_board = 0; /* clear this board so we'll know it's been saved when we next get to it */
		tstack = ostack;
	}
	free(tstack);
	fprintf(reboot_draughts_file, "\n");

	/* actually save it */
	fflush(reboot_draughts_file);
	fclose(reboot_draughts_file);
}

char *reboot_load_draughts_game(char *string) {
	int x,y;
	char *next, *pc, *p2c;
	player *p, *p2;
	draughts_board *board;

	/* Find the players */
	next = strchr(string, ' '); *next++ = 0;
	pc = string;
	p = find_player_absolute_quiet(string);

	string = next; next = strchr(string, ' '); *next++ = 0;
	p2c = string;
	p2 = find_player_absolute_quiet(string);

	/* Initialise the board */
	board = draughts_board_create(p, p2);

	/*  Load the board */
	string = next; next = strchr(string, '\n'); *next++ = 0;

	if (*string++ == '1')
  		board->next = board->a;
  	else
  		board->next = board->b;

  	if (*string == '0')
  		board->prev = 0;
  	else if (*string == '1')
  		board->prev = board->a;
  	else
  		board->prev = board->b;
	string++;

	board->moved = *string++ - '0';
	board->oxs = *string++ - '0';
	board->oys = *string++ - '0';
	board->oxt = *string++ - '0';
	board->oyt = *string++ - '0';
  	for (x=0; x<8; x++) {
		for (y=0; y<8; y++) {
			board->square[x][y] = *string++ - '0';
		}
	}

	string = next;
	/* In case someone has been lost in the reconnection... */
	if (!p && !p2) {
		/* Neither player not found, abandon the game */
		free(board);
		return string;
	} else if (!p || !p2) {
		/* Only one missing */
		if (p) {
			if (*p2c) TELLPLAYER(p, " %s has been disconnected.\n", p2c);
			p->draughts_board = board;
		} else if (p2) {
			if (*pc) TELLPLAYER(p2, " %s has been disconnected.\n", pc);
			p2->draughts_board = board;
		}
	} else {
		p->draughts_board = board;
		p2->draughts_board = board;
	}

	return string;
}

void reboot_load_draughts(void) {
	char *string;

	file reboot_draughts_file = load_file("junk/rebooting/draughts");

	if (!(reboot_draughts_file.where)) {
		log("reboot", "Couldn't open junk/rebooting/draughts for reading");
		return;
	}

	string = reboot_draughts_file.where;
	while (string && *string != '\n') {
		string = reboot_load_draughts_game(string);
	}
}



/********************/
/*** Version code ***/
/********************/

void draughts_version(void) {
  stack += sprintf(stack, " -=*> Draughts v1.1 (by radiac) installed\n");
}

