/*
 * Playground+ - map.c
 * Dynamic Map code v1.0 (by Radiac)
 *    http://radiac.net/pgplus
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "include/config.h"
#include "include/player.h"
#include "include/proto.h"

map_col *map_col_start = 0;
char *map_string = 0;

room *map_find_room(char *str)
{
  char *scan, *oldstack;
  saved_player *sp;
  room *r;

  oldstack = stack;
  scan = str;
  while (*scan && *scan != '.')
    *stack++ = *scan++;
  if (!*scan)
  {
    /* Invalid format */
    sprintf(oldstack, "Invalid room '%s' - invalid format", str);
    stack = end_string(oldstack);
    log("error", oldstack);
    stack = oldstack;
    return 0;
  }
  *stack++ = 0;
  scan++;
  if (!*oldstack) {
    /* No room owner specified */
    sprintf(oldstack, "Invalid room '%s' - no room owner specified", str);
    stack = end_string(oldstack);
    log("error", oldstack);
    stack = oldstack;
    return 0;
  }
  lower_case(oldstack);
  sp = find_saved_player(oldstack);
  if (!sp) {
    /* owner not found */
    sprintf(oldstack, "Invalid room '%s' - owner not found", str);
    stack = end_string(oldstack);
    log("error", oldstack);
    stack = oldstack;
    return 0;
  }
  /* take a look at the room */
  strcpy(oldstack, scan);
  lower_case(oldstack);
  stack = end_string(oldstack);
  r = sp->rooms;
  while (r)
  {
    strcpy(stack, r->id);
    lower_case(stack);
    if (!strcmp(stack, oldstack))
    {
      stack = oldstack;
      current_room = r;
      if (!decompress_room(r))
	return 0;
      return r;
    }
    if (!sp)
    {
      /* Cant find player for room */
      sprintf(oldstack, "Invalid room '%s' - can't find player for room", str);
      stack = end_string(oldstack);
      log("error", oldstack);
      stack = oldstack;
      return 0;
    }
    strcpy(oldstack, scan);
    lower_case(oldstack);
    stack = end_string(oldstack);
    r = sp->rooms;
    while (r)
    {
      strcpy(stack, r->id);
      lower_case(stack);
      if (!strcmp(stack, oldstack))
      {
        stack = oldstack;
        current_room = r;
        if (!decompress_room(r))
          return 0;
        return r;
      }
      r = r->next;
    }
    /* Can't find the room */
    sprintf(oldstack, "Invalid room '%s' - can't find room", str);
    stack = end_string(oldstack);
    log("error", oldstack);
    stack = oldstack;
    return 0;
  }
  return r;
}

void init_map(void) {
	char *oldstack, *line;
	file f;
	map_col *col, *prev;
	prev = 0;

	oldstack = stack;
	map_col_start = 0;
	col = 0;

	f = load_file("files/map.msg");

	line = get_line(&f); /* from rooms.c */
	while (line && *line && strcasecmp(line, "end")) {
		/* Check its a valid colour definition  */
		if (*(line+3) != ' ' || !*(line+4)) {
			log("error","Invalid colour definitions in files/map.msg. Map unavailable.");
			map_col_start = 0;
			return;
		}
		col = (map_col *)malloc(sizeof(map_col));
		if (!map_col_start)
			map_col_start = col;
		col->id = *line;
		col->selected = *(line+1);
		col->unselected = *(line+2);

		line+=4; /* skip to room name */
		col->room = map_find_room(line);
		if (!col->room) {
			/* Invalid map */
			log("error", "Invalid map in 'map.msg' - map will be unavailable.");
			map_col_start = 0;
			return;
		}

		if (prev)
			prev->next = col;
		prev = col;
		line = get_line(&f); /* get next line */
	}
	if (col)
		col->next = 0;

	/* do something with the rest */
	map_string = (char *)malloc(f.length);
	strcpy(map_string, f.where);
}


void map(player * p, char * str) {
	/* str not used yet, but will be */
	char *temp_map_string, *tms_start, col_replace;
	map_col *col_scan;

	temp_map_string = (char *)malloc(strlen(map_string));
	strcpy(temp_map_string, map_string);

	if (!map_col_start) {
		TELLPLAYER(p, "Sorry, but there seems to be a problem with the map at the moment");
		return;
	}

	tms_start = temp_map_string;
	col_scan = map_col_start;

	for (col_scan = map_col_start; col_scan; col_scan = col_scan->next) {
		if (p->location == col_scan->room) {
			col_replace = col_scan->selected;
		} else {
			col_replace = col_scan->unselected;
		}
		while (*(temp_map_string+1)) {
			if (*temp_map_string == '^' && *(temp_map_string+1) == col_scan->id) {
				/* This is one */
				*(temp_map_string+1) = col_replace;
			}
			temp_map_string++;
		}
		temp_map_string = tms_start;
	}
	TELLPLAYER(p, "%s", temp_map_string);
}

void map_version(void)
{
  stack += sprintf(stack, " -=*> Dynamic Map v1.0 (by radiac) installed\n");
}

